function cptool( filename )
% cptool - a GUI tool for analyzing a sequence of states in the classic
%   cart and pole control problem. Usage should be fairly self-explanatory,
%   but documentation under the Help menu will probably be added
%   eventually. The cptool window is currently not resizeable but, pending
%   difficulties in handling it, will be. For convenience, here are some
%   key points for using cptool:
%
%   - The name of a file to load can be given as an argument to cptool,
%     though this is not required. Syntax: cptool( FILENAME ) , where
%     FILENAME is the file to be loaded.
% 
%   - Each line of the input file should have the following format:
%
%       X X_DOT THETA THETA_DOT ACTION
%
%       A line can be created (assuming your file pointer is fout) by:
%   
%       fprintf( fout, '%f %f %f %f %f\n', state', action );
%
%   - "Save Image" under the File menu will save a copy of the currently
%     displayed figure (whether a scene of the animation, an x curve, etc.)
%     using a given file name; currently, only the PNG format is supported.
%     NOTE: This function is not yet implemented. (stay tuned)
%
%   - Tau is the amount of time which passes between each state (also
%     sometimes called the "timestep"). See cart_pole.m for details.
%
%   - Pole Length is the length of the pole. Note that the environment
%     parameter in cart_pole.m called length is half of the pole length.
%     See cart_pole.m for details.
%
%   - It should be kept in mind that cptool fills a memory buffer with the
%     entire episode read from an input file. This might prove difficult
%     for very large data sets. Please report any difficulties to Scott at
%     slivingston@caltech.edu.
%
%
% Written by Scott Livingston, October 2007
%
% Machine Intelligence Lab, EECS - Univ. of Tennessee
%
% Released under the GNU General Public License, version 3, see
% GPLv3.txt or README for details.


% initialize appropriate variables:
if ~exist('filename','var') % filename -> file to read data from
    filename = '';
end

current_step = -1; % current step (based on discrete time); -1 indicates no
                   % data has been loaded.
sa_data = [];      % matrix that holds buffer of all state-action data (as
                   % read from input file); see detailed notes below.
tau = 0.02;        % time between states (i.e. transition time) (in s)
pole_length = 1.0; % length of the pole (in m)
Xmin = -5;         % axis bounds
Xmax = 5;
Ymin = 0;
Ymax = 10;

% Notes regarding the format of sa_data. It is an n x 5 matrix where row
% number represents step number (plus one) and each row follows the
% following layout:
%
% [ x x_dot theta theta_dot force ]
%
% where the vector (x, x_dot, theta, theta_dot) is the state, and
% force is the action.

% create the GUI window
hfig = figure( 'Visible', 'off', 'MenuBar', 'none', ...
               'Name', 'cptool - <no file loaded>', ...
               'NumberTitle', 'off', 'Resize', 'off', 'Toolbar', 'none', ...
               'Position', [100, 500, 700, 450], ...
               'DeleteFcn', {@cleanup_cptool_Callback} );

% use standard system background color as GUI background
defaultBG = get( 0, 'defaultUicontrolBackgroundColor' );
set( hfig, 'Color', defaultBG );

% create the menu
hm_file = uimenu( hfig, 'Label', 'File' );
    hm_open = uimenu( hm_file, 'Label', 'Open', 'Callback', {@file_open_Callback} );
    hm_reload = uimenu( hm_file, 'Label', 'Reload', 'Callback', {@file_reload_Callback} );
    hm_save = uimenu( hm_file, 'Label', 'Save Image' );
    hm_close = uimenu( hm_file, 'Label', 'Close', 'Callback', {@file_close_Callback} );
hm_help = uimenu( hfig, 'Label', 'Help' );
    hm_units = uimenu( hm_help, 'Label', 'System Units', 'Callback', {@help_units_Callback} );
    hm_about = uimenu( hm_help, 'Label', 'About cptool', 'Callback', {@help_about_Callback} );


% user control and feedback:

% panels
hp_axisctl = uipanel( hfig, 'Title', 'Axis Control', 'Units', 'pixels', ...
                      'Position', [10, 385, 210, 55] );
hp_envctl = uipanel( hfig, 'Title', 'Environment Parameters', ...
                     'Units', 'pixels', 'Position', [500, 385, 190, 55] );
hp_saview = uipanel( hfig, 'Title', 'Current State-Action Pair', ...
                     'Units', 'pixels', 'Position', [500, 100, 190, 275] );
    hp_state = uipanel( 'Parent', hp_saview, 'Title', 'State', ...
                        'Units', 'pixels', 'Position', [5, 140, 180, 85] );
    hp_action = uipanel( 'Parent', hp_saview, 'Title', 'Action', ...
                         'Units', 'pixels', 'Position', [5, 90, 180, 37] );

% axis controls
hst_xmin = uicontrol( hp_axisctl, 'Style', 'text', 'String', 'Xmin:', ...
                      'Position', [5, 20, 35, 15] );
het_xmin = uicontrol( hp_axisctl, 'Style', 'edit', 'String', num2str(Xmin), ...
                      'Position', [45, 20, 45, 15], 'Callback', {@axis_change_Callback} );
hst_xmax = uicontrol( hp_axisctl, 'Style', 'text', 'String', 'Xmax:', ...
                      'Position', [110, 20, 35, 15] );
het_xmax = uicontrol( hp_axisctl, 'Style', 'edit', 'String', num2str(Xmax), ...
                      'Position', [150, 20, 45, 15], 'Callback', {@axis_change_Callback} );
hst_ymin = uicontrol( hp_axisctl, 'Style', 'text', 'String', 'Ymin:', ...
                      'Position', [5, 1, 35, 15] );
het_ymin = uicontrol( hp_axisctl, 'Style', 'edit', 'String', num2str(Ymin), ...
                      'Position', [45, 1, 45, 15], 'Callback', {@axis_change_Callback} );
hst_ymax = uicontrol( hp_axisctl, 'Style', 'text', 'String', 'Ymax:', ...
                      'Position', [110, 1, 35, 15] );
het_ymax = uicontrol( hp_axisctl, 'Style', 'edit', 'String', num2str(Ymax), ...
                      'Position', [150, 1, 45, 15], 'Callback', {@axis_change_Callback} );

% environment parameters %INCREASE WIDTH OF STATIC TEXT FOR "POLE LENGTH"
hst_tau = uicontrol( hp_envctl, 'Style', 'text', 'String', 'Tau:', ...
                     'Position', [5, 20, 70, 15] );
het_tau = uicontrol( hp_envctl, 'Style', 'edit', 'String', num2str(tau), ...
                     'Position', [75, 20, 65, 15], ...
                     'Callback', {@env_change_Callback} );
hst_pole_length = uicontrol( hp_envctl, 'Style', 'text', 'String', 'Pole Length:', ...
                             'Position', [5, 1, 70, 15] );
het_pole_length = uicontrol( hp_envctl, 'Style', 'edit', 'String', num2str(pole_length), ...
                             'Position', [75, 1, 65, 15], ...
                             'Callback', {@env_change_Callback} );

% state-action information for this step number
hst_t = uicontrol( hp_saview, 'Style', 'text', 'String', 't =', ...
                   'Position', [5, 235, 55, 15] );
hst_t_val = uicontrol( hp_saview, 'Style', 'text', 'String', '0', ...
                       'Position', [65, 235, 65, 15] );
hst_x = uicontrol( hp_state, 'Style', 'text', 'String', 'x =', ...
                   'Position', [5, 55, 55, 15] );
hst_x_val = uicontrol( hp_state, 'Style', 'text', 'String', '0', ...
                       'Position', [65, 55, 65, 15] );
hst_x_dot = uicontrol( hp_state, 'Style', 'text', 'String', 'x_dot =', ...
                       'Position', [5, 40, 55, 15] );
hst_x_dot_val = uicontrol( hp_state, 'Style', 'text', 'String', '0', ...
                           'Position', [65, 40, 65, 15] );
hst_theta = uicontrol( hp_state, 'Style', 'text', 'String', 'theta =', ...
                       'Position', [5, 22, 55, 15] );
hst_theta_val = uicontrol( hp_state, 'Style', 'text', 'String', '0', ...
                           'Position', [65, 22, 65, 15] );
hst_theta_dot = uicontrol( hp_state, 'Style', 'text', 'String', 'theta_dot =', ...
                           'Position', [5, 5, 55, 15] );
hst_theta_dot_val = uicontrol( hp_state, 'Style', 'text', 'String', '0', ...
                              'Position', [65, 5, 65, 15] );
hst_force = uicontrol( hp_action, 'Style', 'text', 'String', 'force =', ...
                       'Position', [5, 5, 55, 15] );
hst_force_val = uicontrol( hp_action, 'Style', 'text', 'String', '0', ...
                           'Position', [65, 5, 65, 15] );
                         
% slider and text box (for time step selection)
hs_stepctl = uicontrol( hfig, 'Style', 'slider', 'Min', 0, 'Max', 100, ...
                       'Value', 0, 'SliderStep', [0.01 0.1], ...
                       'Position', [10, 10, 340, 20], 'Callback', {@slider_Callback} );
hst_steplabel = uicontrol( hfig, 'Style', 'text', 'String', 'Step #: ', ...
                           'Position', [375, 10, 50, 20] );
het_stepctl = uicontrol( hfig, 'Style', 'edit', 'String', '0', ...
                         'Position', [425, 10, 65, 20], ...
                         'Callback', {@step_box_Callback} );
% pop-up menu for graph type selection
hpm_graphctl = uicontrol( hfig, 'Style', 'popupmenu', 'Position', [500, 60, 190, 20], ...
                 'String', {'animation','x - plot', 'x_dot - plot', 'theta - plot', 'theta_dot - plot', 'force - plot'}, ...
                 'Value', 1, 'Callback', {@graph_change_Callback} );

% animation control (play/pause button)
htb_playctl = uicontrol( hfig, 'Style', 'togglebutton', ...
                      'String', 'PLAY', 'Value', 0, ...
                      'Position', [572, 10, 46, 25], 'Callback', {@play_button_Callback} );

% create the axes
ha = axes( 'Parent', hfig, 'Units', 'pixels', ...
           'Position', [60, 80, 430, 295] );
axis( ha, [Xmin Xmax Ymin Ymax] );
       
set( hfig, 'Visible', 'on' ); % now, present the cptool window

% now, if a file name was given to cptool as an argument, attempt
% to load it.
if any(size(filename)), file_reload_Callback; end

    
% Callback functions:

    function file_open_Callback( hsrc, event_data )
        % open and read from a new data file
        
        % backup filename (in case an error occurs during loading)
        filename_backup = filename;
        
        filename = uigetfile( {'*.dat'; '*.txt'} ); % open standard file dialog box
        
        if filename == 0 % if no file was selected
            return
        end
        
        % attempt to open the file for reading; file should be formatted
        % as a basic ASCII text file.
        infile = fopen( filename, 'rt' );
        if infile == -1
            errordlg( sprintf( 'Error: could not open %s for reading.', filename ), 'File Loading Error', 'modal' );
            filename = filename_backup; % revert to previous
            return
        end

        % make backup of sa_data (WARNING: this routine may be too memory
        % intensive for some applications or computer systems).
        sa_data_backup = sa_data;
        
        % now, read entire file into sa_data
        sa_data = fscanf( infile, '%f %f %f %f %f', [5 inf] );
        sa_data = sa_data'; % this operation might be very inefficient
                            % (requires further investigation).

        [r,c] = size(sa_data); % verify matrix size
        if r == 0 || c ~= 5 % if invalid
            errordlg( sprintf( 'Error: format of input file is incorrect: detected size is %d x %d.', r, c ), 'File Loading Error', 'modal' );
            fclose( infile ); % cleanup
            filename = filename_backup; % revert to previous
            sa_data = sa_data_backup;
            return
        end
        
        % everything seems to be be fine, close input file
        fclose( infile );
        
        set( hfig, 'Name', sprintf( 'cptool - %s', filename ) ); % update GUI title
        current_step = 0; % default to initial step number
        set( hs_stepctl, 'Max', r-1, ... % adjust slider range according to new data set
             'SliderStep', [1/r min(10/r,.5)], 'Value', 0 );
        set( het_stepctl, 'String', '0' ); % reset step text box
        
        graph_change_Callback( hpm_graphctl, 0 ); % force axis range auto-adjustment
        refresh_all_static; % refresh all GUI information
        
    end

    function file_reload_Callback( hsrc, event_data )
        % reload data from previously open file
        
        % verify file called filename still exists
        if ~exist( filename, 'file' )
            errordlg( sprintf( 'Error: input file %s no longer exists.', filename ), 'File Loading Error', 'modal' );
            return
        end
        
        % attempt to open the file for reading; file should be formatted
        % as a basic ASCII text file.
        infile = fopen( filename, 'rt' );
        if infile == -1
            errordlg( sprintf( 'Error: could not open %s for reading.', filename ), 'File Loading Error', 'modal' );
            return
        end

        % make backup of sa_data (WARNING: this routine may be too memory
        % intensive for some applications or computer systems).
        sa_data_backup = sa_data;
        
        % now, read entire file into sa_data
        sa_data = fscanf( infile, '%f %f %f %f %f', [5 inf] );
        sa_data = sa_data'; % this operation might be very inefficient
                            % (requires further investigation).

        [r,c] = size(sa_data); % verify matrix size
        if r == 0 || c ~= 5 % if invalid
            errordlg( sprintf( 'Error: format of input file is incorrect: detected size is %d x %d.', r, c ), 'File Loading Error', 'modal' );
            fclose( infile ); % cleanup
            sa_data = sa_data_backup; % revert to previous
            return
        end
        
        % everything seems to be be fine, close input file
        fclose( infile );

        current_step = 0; % default to initial step number
        set( hs_stepctl, 'Max', r-1, ... % adjust slider range according to new data set
             'SliderStep', [1/r min(10/r,.5)], 'Value', 0 );
        set( het_stepctl, 'String', '0' ); % reset step text box
        
        graph_change_Callback( hpm_graphctl, 0 ); % force axis range auto-adjustment
        refresh_all_static; % refresh all GUI information
        set( hfig, 'Name', sprintf( 'cptool - %s', filename ) ); % update GUI title
    end

    function help_about_Callback( hsrc, event_data )
        message = ['cptool - a GUI tool for analyzing a sequence of states in the ' ...
                  'classic cart and pole control problem.\n\n' ...
                  'Written by Scott Livingston <slivingston@caltech.edu>, Oct 2007.\n' ...
                  'Univ. of Tennessee, EECS Dept. - Machine Intelligence Lab (http://mil.engr.utk.edu)'];
        msgbox( sprintf(message), 'About cptool' );
    end

    function help_units_Callback( hsrc, event_data )
        message = ['Units used in the cart-pole system:\n\n' ...
                   'Tau  -->  seconds\n' ...
                   'Pole Length  -->  meters\n' ...
                   't  -->  seconds\n' ...
                   'x  -->  meters\n' ...
                   'x_dot  -->  meters/second\n' ...
                   'theta  -->  radians\n' ...
                   'theta_dot  -->  radians/second\n' ...
                   'force  -->  Newtons\n' ...
                   'Step  -->  unitless (corresponds to a state transition in discrete time)\n'];
        msgbox( sprintf(message), 'Units in cptool' );
    end

    function slider_Callback( hsrc, event_data )
        % if no file is loaded (current_step = -1), then reset to 0;
        % otherwise, set current_step to value of slider.
        if current_step == -1
            set( hs_stepctl, 'Value', 0 );
        else
            current_step = round(get( hs_stepctl, 'Value' ));
            set( hs_stepctl, 'Value', current_step );
            set( het_stepctl, 'String', num2str(current_step) );
            refresh_all_static; % refresh GUI information
        end
    end

    function step_box_Callback( hsrc, event_data )
        % if no file is loaded (current_step = -1), then reset to 0;
        % otherwise, verify desired step is valid, and if so, set
        % current_step to value of text box.
        
        if current_step == -1
            set( het_stepctl, 'String', '0' );
        else
            temp = current_step; % temporary copy of current step number
            current_step = str2num( get( het_stepctl, 'String' ) );
            [episode_len,c] = size( sa_data );
            if isempty( current_step ) ... % entry is not a number
               || current_step-floor(current_step) ~= 0 ... % entry is not an integer
               || current_step < 0 || current_step > episode_len-1 % entry is outside valid range
                current_step = temp;
            end
            set( het_stepctl, 'String', num2str(current_step) );
            set( hs_stepctl, 'Value', current_step );
        end
        
        refresh_all_static; % refresh GUI information
    end

    function axis_change_Callback( hsrc, event_data )
        % handle a change in the axis ranges;
        % if input is invalid, reset value to previous.
        switch hsrc
            case het_xmin
                temp = Xmin;
                Xmin = str2num(get( hsrc, 'String' ));
                if isempty( Xmin ) || Xmin > Xmax
                    Xmin = temp;
                    set( hsrc, 'String', num2str(Xmin) );
                end
            case het_xmax
                temp = Xmax;
                Xmax = str2num(get( hsrc, 'String' ));
                if isempty( Xmax ) || Xmax < Xmin
                    Xmax = temp;
                    set( hsrc, 'String', num2str(Xmax) );
                end
            case het_ymin
                temp = Ymin;
                Ymin = str2num(get( hsrc, 'String' ));
                if isempty( Ymin ) || Ymin > Ymax
                    Ymin = temp;
                    set( hsrc, 'String', num2str(Ymin) );
                end
            case het_ymax
                temp = Ymax;
                Ymax = str2num(get( hsrc, 'String' ));
                if isempty( Ymax ) || Ymax < Ymin
                    Ymax = temp;
                    set( hsrc, 'String', num2str(Ymax) );
                end
            otherwise
                errordlg( 'Error: axis_change_Callback() received a bad argument.', 'Bad Function Argument', 'modal' );
                return
        end
        
        % now, update the axes and graph
        axis( ha, [Xmin Xmax Ymin Ymax] );
        draw_graph;
    end

    function env_change_Callback( hsrc, event_data )
        % handle change in environmental parameters
        
        if hsrc == het_tau % tau value was changed
            temp = tau;
            tau = str2num( get( het_tau, 'String' ) );
            if isempty( tau ) || tau < 0 % if input is invalid
                tau = temp; % reset tau to original value
                set( het_tau, 'String', num2str(tau) );
            end
        elseif hsrc == het_pole_length % pole length was changed
            temp = pole_length;
            pole_length = str2num( get( het_pole_length, 'String' ) );
            if isempty( pole_length ) || pole_length < 0 % if input is invalid
                pole_length = temp; % reset pole_length to original value
                set( het_pole_length, 'String', num2str(pole_length) );
            end
        end
        
        % refresh the GUI given new parameter information
        refresh_all_static;
    end

    function play_button_Callback( hsrc, event_data )
        % handle operation of play button;
        % also, perform animation while requested.

        if get( htb_playctl, 'Value' ) % if "play" is requested
            set( htb_playctl, 'String', 'PAUSE' );
            [episode_len,c] = size( sa_data ); % determine length of episode

            % perform animation as long as button remains in "play" position
            tic;
            while current_step < episode_len && get( htb_playctl, 'Value' )
                while toc < tau, end % update according to real time
                set( het_stepctl, 'String', num2str(current_step) ); % refresh step editing box
                set( hs_stepctl, 'Value', current_step ); % refresh step slider
                refresh_all_static;
                drawnow; % force figure refresh
                tic;
                current_step = current_step+1;
            end
            current_step = episode_len-1;

            if current_step == episode_len-1 % unset "play" button
                set( htb_playctl, 'String', 'PLAY', 'Value', 0 );
            end

        else % else, "pause" is requested
            set( htb_playctl, 'String', 'PLAY' );
        end
    end

    function graph_change_Callback( hsrc, event_data )
        % handle change in graph selection
        graph_mode = get( hpm_graphctl, 'Value' );
        % graph modes:
        % 1 -> animation
        % 2 -> x - plot
        % 3 -> x_dot - plot
        % 4 -> theta - plot
        % 5 -> theta_dot - plot
        % 6 -> force - plot
        
        % in graph modes others than animation, disable step slider and
        % text box.
        if graph_mode == 1
            set( hs_stepctl, 'Enable', 'on' );
            set( het_stepctl, 'Enable', 'on' );
            set( htb_playctl, 'Enable', 'on' );
        else
            set( hs_stepctl, 'Enable', 'off' );
            set( het_stepctl, 'Enable', 'off' );
            set( htb_playctl, 'Enable', 'off' );
        end

	% if no data is loaded, return without making changes to figure
        if current_step == -1 || isempty( sa_data )
            return
        end
        
        % make axis adjustments common to all non-animation graph modes
        [episode_len,c] = size( sa_data ); % find total number of steps
        if graph_mode > 1 && graph_mode <= 6
            Xmin = 0;
            Xmax = episode_len-1;
        end
        
        % make adjustments to axis ranges for each graph mode
        if graph_mode == 1 % animation (currently only shows step images)
            Xmin = min( sa_data( :, 1 ) ) - pole_length;
            Xmax = max( sa_data( :, 1 ) ) + pole_length;
            Ymin = 0;
            Ymax = abs(Xmax-Xmin)*295/430; % adjust ranges to same scale;
                                           % this is based on size of axes
                                           % in pixels.
            
        elseif graph_mode == 2 % x - plot
            Ymax = max( sa_data( :, 1 ) );
            Ymin = min( sa_data( :, 1 ) );
            Ymin = Ymin - 0.1*abs(Ymax-Ymin);
            Ymax = Ymax + 0.1*abs(Ymax-Ymin);
            
        elseif graph_mode == 3 % x_dot - plot
            Ymax = max( sa_data( :, 2 ) );
            Ymin = min( sa_data( :, 2 ) );
            Ymin = Ymin - 0.1*abs(Ymax-Ymin);
            Ymax = Ymax + 0.1*abs(Ymax-Ymin);
            
        elseif graph_mode == 4 % theta - plot
            Ymax = max( sa_data( :, 3 ) );
            Ymin = min( sa_data( :, 3 ) );
            Ymin = Ymin - 0.1*abs(Ymax-Ymin);
            Ymax = Ymax + 0.1*abs(Ymax-Ymin);
            
        elseif graph_mode == 5 % theta_dot - plot
            Ymax = max( sa_data( :, 4 ) );
            Ymin = min( sa_data( :, 4 ) );
            Ymin = Ymin - 0.1*abs(Ymax-Ymin);
            Ymax = Ymax + 0.1*abs(Ymax-Ymin);
            
        elseif graph_mode == 6 % force - plot
            Ymax = max( sa_data( :, 5 ) );
            Ymin = min( sa_data( :, 5 ) );
            Ymin = Ymin - 0.1*abs(Ymax-Ymin);
            Ymax = Ymax + 0.1*abs(Ymax-Ymin);
            
        elseif graph_mode ~= 1
            errordlg( sprintf( 'Error: graph mode %d is invalid.', graph_mode ), 'Graph Update Error', 'modal' );
            return
        end
        
        % update uicontrol text boxes for axis ranges
        set( het_xmin, 'String', num2str(Xmin) );
        set( het_xmax, 'String', num2str(Xmax) );
        set( het_ymin, 'String', num2str(Ymin) );
        set( het_ymax, 'String', num2str(Ymax) );
        
        draw_graph; % since draw_graph() reads directly from the selection
                    % pop-up menu, no argument is needed.
    end

    function refresh_all_static
        % refresh display of all static information, such as horizontal
        % position x, including the graph.
        
        % if data set is empty (i.e. no file has been loaded), ignore
        % refresh request.
        if current_step == -1, return; end;
        
        % update continuous time corresponding to this (current) step
        set( hst_t_val, 'String', num2str( tau*current_step ) );
        
        % update static text information for state and action variables
        % given current step number (that is, discrete time position).
        set( hst_x_val, 'String', num2str( sa_data( current_step+1, 1 ) ) );
        set( hst_x_dot_val, 'String', num2str( sa_data( current_step+1, 2 ) ) );
        set( hst_theta_val, 'String', num2str( sa_data( current_step+1, 3 ) ) );
        set( hst_theta_dot_val, 'String', num2str( sa_data( current_step+1, 4 ) ) );
        set( hst_force_val, 'String', num2str( sa_data( current_step+1, 5 ) ) );        

        draw_graph; % redraw the graph
    end

    function draw_graph
        % use state, action, step and environment parameters to update the
        % graph.
        
        cla( ha ); % clear the axes
        
        % nothing to draw if sa_data is empty
        if isempty( sa_data )
            return
        end
        
        graph_mode = get( hpm_graphctl, 'Value' );
        % (see notes in graph_change_Callback() definition)
        
        [episode_len,c] = size( sa_data ); % find total number of steps in episode
        
        if graph_mode == 1 % animation (currently only shows step images)

            cart_height = 0.25; % used in drawing the cart and pole
            cart_width = 0.5;
            pole_vertices = [sa_data( current_step+1, 1 )+pole_length/8*sin(sa_data( current_step+1, 3 )-pi/8) cart_height+pole_length/8*cos(sa_data( current_step+1, 3 )-pi/8); ...
                             sa_data( current_step+1, 1 ) cart_height; ...
                             sa_data( current_step+1, 1 )+pole_length/8*sin(sa_data( current_step+1, 3 )+pi/8) cart_height+pole_length/8*cos(sa_data( current_step+1, 3 )+pi/8); ...
                             sa_data( current_step+1, 1 )+pole_length*sin(sa_data( current_step+1, 3 ))-pole_length/8*sin(sa_data( current_step+1, 3 )-pi/8) cart_height+pole_length*cos(sa_data( current_step+1, 3 ))-pole_length/8*cos(sa_data( current_step+1, 3 )-pi/8); ...
                             sa_data( current_step+1, 1 )+pole_length*sin(sa_data( current_step+1, 3 )) cart_height+pole_length*cos(sa_data( current_step+1, 3 )); ...
                             sa_data( current_step+1, 1 )+pole_length*sin(sa_data( current_step+1, 3 ))-pole_length/8*sin(sa_data( current_step+1, 3 )+pi/8) cart_height+pole_length*cos(sa_data( current_step+1, 3 ))-pole_length/8*cos(sa_data( current_step+1, 3 )+pi/8)];
            hinge_res = 0.1; % generate vertices for the hinge;
                             % (where the pole connects to the cart)
            hinge_radius = cart_width/20;
            hinge_X = sa_data( current_step+1, 1 )+hinge_radius*cos(0:hinge_res:2*pi);
            hinge_Y = cart_height+hinge_radius*sin(0:hinge_res:2*pi);

            patch( [sa_data( current_step+1, 1 )-cart_width/2 sa_data( current_step+1, 1 )-cart_width/2 sa_data( current_step+1, 1 )+cart_width/2 sa_data( current_step+1, 1 )+cart_width/2], ...
                   [0 cart_height cart_height 0], ...
                   [0.5 0.5 1.0] ); % the cart
            patch( pole_vertices(:,1), pole_vertices(:,2), [.5 1.0 .5] ); % the pole
            
            patch( hinge_X, hinge_Y, [0 0 0] ); % the hinge
            
            axis( ha, [Xmin Xmax Ymin Ymax] ); % apply axis ranges
            xlabel( 'x (horizontal position)' );
            ylabel( '' );
            
        elseif graph_mode == 2 % x - plot
            
            plot( ha, 0:(episode_len-1), sa_data( :, 1 ) ); % make plot
            axis( ha, [Xmin Xmax Ymin Ymax] ); % apply axis ranges
            xlabel( 'Step Number (discrete)' );
            ylabel( 'position x (in m)' );
            
        elseif graph_mode == 3 % x_dot - plot
            
            plot( ha, 0:(episode_len-1), sa_data( :, 2 ) ); % make plot
            axis( ha, [Xmin Xmax Ymin Ymax] ); % apply axis ranges
            xlabel( 'Step Number (discrete)' );
            ylabel( 'cart velocity x\_dot (in m/s)' );
            
        elseif graph_mode == 4 % theta - plot
            
            plot( ha, 0:(episode_len-1), sa_data( :, 3 ) ); % make plot
            axis( ha, [Xmin Xmax Ymin Ymax] ); % apply axis ranges
            xlabel( 'Step Number (discrete)' );
            ylabel( 'pole angle theta (in radians)' );
            
        elseif graph_mode == 5 % theta_dot - plot
            
            plot( ha, 0:(episode_len-1), sa_data( :, 4 ) ); % make plot
            axis( ha, [Xmin Xmax Ymin Ymax] ); % apply axis ranges
            xlabel( 'Step Number (discrete)' );
            ylabel( 'pole angle velocity theta\_dot (in rad/s)' );
            
        elseif graph_mode == 6 % force - plot
            
            plot( ha, 0:(episode_len-1), sa_data( :, 5 ) ); % make plot
            axis( ha, [Xmin Xmax Ymin Ymax] ); % apply axis ranges
            xlabel( 'Step Number (discrete)' );
            ylabel( 'applied force (in N)' );

        else
            errordlg( sprintf( 'Error: graph mode %d is invalid.', graph_mode ), 'Graph Update Error', 'modal' );
            return
        end
                
    end

    function file_close_Callback( hsrc, event_data )
        delete( hfig ); % close the cptool GUI
    end

    function cleanup_cptool_Callback( hsrc, event_data )
        % perform necessary cleanup operations before cptool GUI exits
        % currently, there are none!
    end


end % end of cptool
